#pragma once

#include <glm/vec2.hpp>
#include <vector>

namespace game {
    class Missile;

    /*
     * Trace of a missile through the space.
     * Useful for rendering sth. like a smoke trail to follow the rocket.
     */
    class Trace {
        public:
            Trace(const Missile *missile);

            // Add the current position of the missile as a new point on the
            // trace.
            // Parameters:
            // forceAdd: set to true to add this point (good for the endpoint of
            // the missile) in case the fidelityCounter would skip the current position.
            void addPointFromMissile(bool forceAdd=false);

            // TODO: add extendLastPointToPosition() method for saving points /
            // optimization later on

            /*
             * Trace point data to be used when rendering.
             */
            struct TracePoint {
                TracePoint(const Missile *missile);

                glm::vec2 position;
                float     speed;
            };

            std::vector<TracePoint> points;
            const Missile *missile; // missile which creates this path.

            // counter which is incremented each time addPointFromMissile() is called.
            // when reaching a certain value the point is saved for
            // optimization.
            int fidelityCounter;
    };
}
